/**
 * org.apache.flex.charts.beads.XAxisBead
 *
 * @fileoverview
 *
 * @suppress {checkTypes}
 */

goog.provide('org.apache.flex.charts.beads.XAxisBead');

goog.require('org.apache.flex.charts.beads.ChartItemRendererFactory');
goog.require('org.apache.flex.core.FilledRectangle');
goog.require('org.apache.flex.events.Event');
goog.require('org.apache.flex.html.Label');
goog.require('org.apache.flex.charts.beads.layouts.BarChartLayout');
goog.require('org.apache.flex.html.beads.models.ArraySelectionModel');
goog.require('org.apache.flex.core.UIBase');
goog.require('org.apache.flex.events.IEventDispatcher');
goog.require('org.apache.flex.core.IBeadLayout');
goog.require('org.apache.flex.core.IStrand');
goog.require('org.apache.flex.core.IBead');
goog.require('org.apache.flex.core.IDataProviderItemRendererMapper');
goog.require('org.apache.flex.charts.core.IChart');
goog.require('org.apache.flex.core.ISelectionModel');
goog.require('org.apache.flex.charts.core.IChartAxis');



/**
 * @constructor
 * @implements {org.apache.flex.core.IBead}
 * @implements {org.apache.flex.charts.core.IChartAxis}
 */
org.apache.flex.charts.beads.XAxisBead = function() {
};


/**
 * @private
 * @type {string}
 */
org.apache.flex.charts.beads.XAxisBead.prototype._labelField;


/**
 * @expose
 * @return {string}
 */
org.apache.flex.charts.beads.XAxisBead.prototype.get_labelField = function() {
  return this._labelField;
};


/**
 * @expose
 * @param {string} value
 */
org.apache.flex.charts.beads.XAxisBead.prototype.set_labelField = function(value) {
  this._labelField = value;
};


/**
 * @private
 * @type {org.apache.flex.core.IStrand}
 */
org.apache.flex.charts.beads.XAxisBead.prototype._strand;


/**
 * @expose
 * @param {org.apache.flex.core.IStrand} value
 */
org.apache.flex.charts.beads.XAxisBead.prototype.set_strand = function(value) {
  this._strand = value;
  org.apache.flex.utils.Language.as(this._strand, org.apache.flex.events.IEventDispatcher, true).addEventListener("layoutComplete", goog.bind(this.handleItemsCreated, this));
};


/**
 * @private
 * @param {org.apache.flex.events.Event} event
 */
org.apache.flex.charts.beads.XAxisBead.prototype.handleItemsCreated = function(event) {
  var /** @type {org.apache.flex.charts.beads.ChartItemRendererFactory} */ charter = org.apache.flex.utils.Language.as(this._strand.getBeadByType(org.apache.flex.core.IDataProviderItemRendererMapper), org.apache.flex.charts.beads.ChartItemRendererFactory);
  var /** @type {org.apache.flex.html.beads.models.ArraySelectionModel} */ model = org.apache.flex.utils.Language.as(this._strand.getBeadByType(org.apache.flex.core.ISelectionModel), org.apache.flex.html.beads.models.ArraySelectionModel);
  var /** @type {Array} */ items;
  if (org.apache.flex.utils.Language.is(model.get_dataProvider(), Array))
    items = org.apache.flex.utils.Language.as(model.get_dataProvider(), Array);
  else
    return;
  var /** @type {Array} */ renderers = charter.get_seriesRenderers();
  var /** @type {Array} */ series = org.apache.flex.utils.Language.as(this._strand, org.apache.flex.charts.core.IChart, true).get_series();
  var /** @type {number} */ xpos = 0;
  var /** @type {org.apache.flex.charts.beads.layouts.BarChartLayout} */ barChartLayout = org.apache.flex.utils.Language.as(this._strand.getBeadByType(org.apache.flex.core.IBeadLayout), org.apache.flex.charts.beads.layouts.BarChartLayout);
  var /** @type {number} */ xAxisHeightOffset = barChartLayout.get_xAxisHeight();
  var /** @type {number} */ useWidth = org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true).get_width() / renderers.length;
  var /** @type {org.apache.flex.core.FilledRectangle} */ horzLine = new org.apache.flex.core.FilledRectangle();
  horzLine.set_fillColor(0x111111);
  horzLine.set_x(0);
  horzLine.set_y(org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true).get_height() - xAxisHeightOffset);
  horzLine.set_height(1);
  horzLine.set_width(org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true).get_width());
  org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true).addElement(horzLine);
  var /** @type {number} */ labelY = org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true).get_height() + 8;
  for (var /** @type {number} */ i = 0; i < items.length; i++) {
    var /** @type {org.apache.flex.html.Label} */ label = new org.apache.flex.html.Label();
    label.set_text(items[i][this.get_labelField()]);
    label.set_x(xpos);
    label.set_y(labelY - xAxisHeightOffset);
    org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true).addElement(label);
    var /** @type {org.apache.flex.core.FilledRectangle} */ tick = new org.apache.flex.core.FilledRectangle();
    tick.set_fillColor(0x111111);
    tick.set_x(xpos + useWidth / 2);
    tick.set_y(org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true).get_height() - xAxisHeightOffset);
    tick.set_width(1);
    tick.set_height(5);
    org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true).addElement(tick);
    var /** @type {org.apache.flex.core.UIBase} */ r = org.apache.flex.utils.Language.as(renderers[i][0], org.apache.flex.core.UIBase, true);
    xpos += useWidth;
  }
};


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
org.apache.flex.charts.beads.XAxisBead.prototype.FLEXJS_CLASS_INFO = { names: [{ name: 'XAxisBead', qName: 'org.apache.flex.charts.beads.XAxisBead'}], interfaces: [org.apache.flex.core.IBead, org.apache.flex.charts.core.IChartAxis] };
