/**
 * org.apache.flex.charts.beads.layouts.BarChartLayout
 *
 * @fileoverview
 *
 * @suppress {checkTypes}
 */

goog.provide('org.apache.flex.charts.beads.layouts.BarChartLayout');

goog.require('org.apache.flex.charts.beads.ChartItemRendererFactory');
goog.require('org.apache.flex.charts.supportClasses.BarChartSeries');
goog.require('org.apache.flex.core.UIBase');
goog.require('org.apache.flex.events.Event');
goog.require('org.apache.flex.events.IEventDispatcher');
goog.require('org.apache.flex.charts.core.IChartItemRenderer');
goog.require('org.apache.flex.core.ILayoutParent');
goog.require('org.apache.flex.core.IBeadLayout');
goog.require('org.apache.flex.core.IStrand');
goog.require('org.apache.flex.core.IDataProviderItemRendererMapper');
goog.require('org.apache.flex.charts.core.IChart');



/**
 * @constructor
 * @implements {org.apache.flex.core.IBeadLayout}
 */
org.apache.flex.charts.beads.layouts.BarChartLayout = function() {
};


/**
 * @private
 * @type {org.apache.flex.core.IStrand}
 */
org.apache.flex.charts.beads.layouts.BarChartLayout.prototype._strand;


/**
 * @expose
 * @param {org.apache.flex.core.IStrand} value
 */
org.apache.flex.charts.beads.layouts.BarChartLayout.prototype.set_strand = function(value) {
  this._strand = value;
  org.apache.flex.utils.Language.as(value, org.apache.flex.events.IEventDispatcher, true).addEventListener("widthChanged", goog.bind(this.changeHandler, this));
  org.apache.flex.utils.Language.as(value, org.apache.flex.events.IEventDispatcher, true).addEventListener("childrenAdded", goog.bind(this.changeHandler, this));
  org.apache.flex.utils.Language.as(value, org.apache.flex.events.IEventDispatcher, true).addEventListener("itemsCreated", goog.bind(this.changeHandler, this));
};


/**
 * @private
 * @type {number}
 */
org.apache.flex.charts.beads.layouts.BarChartLayout.prototype._gap = 20;


/**
 * @expose
 * @return {number}
 */
org.apache.flex.charts.beads.layouts.BarChartLayout.prototype.get_gap = function() {
  return this._gap;
};


/**
 * @expose
 * @param {number} value
 */
org.apache.flex.charts.beads.layouts.BarChartLayout.prototype.set_gap = function(value) {
  this._gap = value;
};


/**
 * @private
 * @type {number}
 */
org.apache.flex.charts.beads.layouts.BarChartLayout.prototype._xAxisHeight = 30;


/**
 * @expose
 * @return {number}
 */
org.apache.flex.charts.beads.layouts.BarChartLayout.prototype.get_xAxisHeight = function() {
  return this._xAxisHeight;
};


/**
 * @expose
 * @param {number} value
 */
org.apache.flex.charts.beads.layouts.BarChartLayout.prototype.set_xAxisHeight = function(value) {
  this._xAxisHeight = value;
};


/**
 * @private
 * @param {org.apache.flex.events.Event} event
 */
org.apache.flex.charts.beads.layouts.BarChartLayout.prototype.changeHandler = function(event) {
  var /** @type {org.apache.flex.core.ILayoutParent} */ layoutParent = org.apache.flex.utils.Language.as(this._strand.getBeadByType(org.apache.flex.core.ILayoutParent), org.apache.flex.core.ILayoutParent);
  var /** @type {org.apache.flex.charts.beads.ChartItemRendererFactory} */ factory = org.apache.flex.utils.Language.as(this._strand.getBeadByType(org.apache.flex.core.IDataProviderItemRendererMapper), org.apache.flex.charts.beads.ChartItemRendererFactory);
  var /** @type {number} */ n = factory.get_seriesRenderers().length;
  var /** @type {number} */ xpos = 0;
  var /** @type {number} */ useWidth = (org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true).get_width() / n) - this.get_gap();
  var /** @type {number} */ useHeight = org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true).get_height() - this.get_xAxisHeight();
  var /** @type {number} */ maxYValue = 0;
  var /** @type {Array} */ series = org.apache.flex.utils.Language.as(this._strand, org.apache.flex.charts.core.IChart, true).get_series();
  org.apache.flex.utils.Language.trace("There are " + series.length + " series in this chart");
  var /** @type {Array} */ seriesMaxes = [];
  for (var /** @type {number} */ s = 0; s < series.length; s++) {
    var /** @type {org.apache.flex.charts.supportClasses.BarChartSeries} */ bcs = org.apache.flex.utils.Language.as(series[s], org.apache.flex.charts.supportClasses.BarChartSeries);
    seriesMaxes.push({maxValue:0, scaleFactor:0});
    for (var /** @type {number} */ i = 0; i < n; i++) {
      var /** @type {Array} */ m = org.apache.flex.utils.Language.as(factory.get_seriesRenderers()[i], Array);
      var /** @type {org.apache.flex.charts.core.IChartItemRenderer} */ item = org.apache.flex.utils.Language.as(m[s], org.apache.flex.charts.core.IChartItemRenderer);
      var /** @type {Object} */ data = item.get_data();
      var /** @type {string} */ field = bcs.get_yField();
      var /** @type {number} */ yValue = Number(data[field]);
      seriesMaxes[s].maxValue = Math.max(seriesMaxes[s].maxValue, yValue);
    }
    seriesMaxes[s].scaleFactor = useHeight / seriesMaxes[s].maxValue;
  }
  for (i = 0; i < n; i++) {
    m = org.apache.flex.utils.Language.as(factory.get_seriesRenderers()[i], Array);
    for (s = 0; s < m.length; s++) {
      var /** @type {number} */ seriesWidth = useWidth / series.length;
      var /** @type {org.apache.flex.charts.core.IChartItemRenderer} */ child = org.apache.flex.utils.Language.as(m[s], org.apache.flex.charts.core.IChartItemRenderer);
      data = child.get_data();
      yValue = Number(data[child.get_yField()]);
      child.set_y(useHeight - yValue * seriesMaxes[s].scaleFactor);
      child.set_x(xpos);
      child.set_width(seriesWidth);
      child.set_height(yValue * seriesMaxes[s].scaleFactor);
      xpos += seriesWidth;
    }
    xpos += this.get_gap();
  }
  org.apache.flex.utils.Language.as(this._strand, org.apache.flex.events.IEventDispatcher, true).dispatchEvent(new org.apache.flex.events.Event("layoutComplete"));
};


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
org.apache.flex.charts.beads.layouts.BarChartLayout.prototype.FLEXJS_CLASS_INFO = { names: [{ name: 'BarChartLayout', qName: 'org.apache.flex.charts.beads.layouts.BarChartLayout'}], interfaces: [org.apache.flex.core.IBeadLayout] };
