/**
 * org.apache.flex.effects.Move
 *
 * @fileoverview
 *
 * @suppress {checkTypes}
 */

goog.provide('org.apache.flex.effects.Move');

goog.require('org.apache.flex.effects.Tween');
goog.require('org.apache.flex.core.IUIBase');
goog.require('org.apache.flex.core.IDocument');



/**
 * @constructor
 * @extends {org.apache.flex.effects.Tween}
 * @implements {org.apache.flex.core.IDocument}
 * @param {org.apache.flex.core.IUIBase=} target
 */
org.apache.flex.effects.Move = function(target) {
  target = typeof target !== 'undefined' ? target : null;
  org.apache.flex.effects.Move.base(this, 'constructor');
  this.target = target;
  this.startValue = 0;
  this.endValue = 1;
  this.listener = this;
};
goog.inherits(org.apache.flex.effects.Move, org.apache.flex.effects.Tween);


/**
 * @private
 * @type {Object}
 */
org.apache.flex.effects.Move.prototype.document;


/**
 * @private
 * @type {org.apache.flex.core.IUIBase}
 */
org.apache.flex.effects.Move.prototype.target;


/**
 * @private
 * @type {number}
 */
org.apache.flex.effects.Move.prototype.dx;


/**
 * @private
 * @type {number}
 */
org.apache.flex.effects.Move.prototype.dy;


/**
 * @private
 * @type {number}
 */
org.apache.flex.effects.Move.prototype.xStart;


/**
 * @private
 * @type {number}
 */
org.apache.flex.effects.Move.prototype.yStart;


/**
 * @type {number}
 */
org.apache.flex.effects.Move.prototype.xFrom;


/**
 * @type {number}
 */
org.apache.flex.effects.Move.prototype.xTo;


/**
 * @type {number}
 */
org.apache.flex.effects.Move.prototype.yFrom;


/**
 * @type {number}
 */
org.apache.flex.effects.Move.prototype.yTo;


/**
 * @expose
 * @param {Object} document
 * @param {string=} id
 */
org.apache.flex.effects.Move.prototype.setDocument = function(document, id) {
  id = typeof id !== 'undefined' ? id : null;
  this.document = document;
};


/**
 * @expose
 * @override
 */
org.apache.flex.effects.Move.prototype.play = function() {
  if (org.apache.flex.utils.Language.is(this.target, String))
    this.target = this.document[this.target];
  if (isNaN(this.xFrom))
    this.xStart = this.target.get_x();
  if (isNaN(this.xTo))
    this.dx = 0;
  else
    this.dx = this.xTo - this.xStart;
  if (isNaN(this.yFrom))
    this.yStart = this.target.get_y();
  if (isNaN(this.yTo))
    this.dy = 0;
  else
    this.dy = this.yTo - this.yStart;
  org.apache.flex.effects.Move.base(this, 'play');
};


/**
 * @expose
 * @param {number} value
 */
org.apache.flex.effects.Move.prototype.onTweenUpdate = function(value) {
  if (this.dx)
    this.target.set_x(this.xStart + value * this.dx);
  if (this.dy)
    this.target.set_y(this.yStart + value * this.dy);
};


/**
 * @expose
 * @param {number} value
 */
org.apache.flex.effects.Move.prototype.onTweenEnd = function(value) {
  if (this.dx)
    this.target.set_x(this.xStart + this.dx);
  if (this.dy)
    this.target.set_y(this.yStart + this.dy);
};


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
org.apache.flex.effects.Move.prototype.FLEXJS_CLASS_INFO = { names: [{ name: 'Move', qName: 'org.apache.flex.effects.Move'}], interfaces: [org.apache.flex.core.IDocument] };
