/**
 * org.apache.flex.html.beads.DataGridView
 *
 * @fileoverview
 *
 * @suppress {checkTypes}
 */

goog.provide('org.apache.flex.html.beads.DataGridView');

goog.require('org.apache.flex.html.Container');
goog.require('org.apache.flex.html.beads.layouts.ButtonBarLayout');
goog.require('org.apache.flex.html.beads.layouts.DataGridLayout');
goog.require('org.apache.flex.events.Event');
goog.require('org.apache.flex.core.BeadViewBase');
goog.require('org.apache.flex.html.ButtonBar');
goog.require('org.apache.flex.html.beads.models.ArraySelectionModel');
goog.require('org.apache.flex.html.supportClasses.DataGridColumn');
goog.require('org.apache.flex.core.UIBase');
goog.require('org.apache.flex.html.List');
goog.require('org.apache.flex.events.IEventDispatcher');
goog.require('org.apache.flex.core.IBead');
goog.require('org.apache.flex.core.IStrand');
goog.require('org.apache.flex.html.beads.IDataGridView');
goog.require('org.apache.flex.core.IDataGridModel');
goog.require('org.apache.flex.core.ISelectionModel');
goog.require('org.apache.flex.core.IBeadModel');
goog.require('org.apache.flex.core.IDataGridLayout');



/**
 * @constructor
 * @extends {org.apache.flex.core.BeadViewBase}
 * @implements {org.apache.flex.html.beads.IDataGridView}
 */
org.apache.flex.html.beads.DataGridView = function() {
  org.apache.flex.html.beads.DataGridView.base(this, 'constructor');
};
goog.inherits(org.apache.flex.html.beads.DataGridView, org.apache.flex.core.BeadViewBase);


/**
 * @private
 * @type {org.apache.flex.html.ButtonBar}
 */
org.apache.flex.html.beads.DataGridView.prototype.buttonBar;


/**
 * @private
 * @type {org.apache.flex.html.beads.models.ArraySelectionModel}
 */
org.apache.flex.html.beads.DataGridView.prototype.buttonBarModel;


/**
 * @private
 * @type {org.apache.flex.html.Container}
 */
org.apache.flex.html.beads.DataGridView.prototype.columnContainer;


/**
 * @private
 * @type {Array}
 */
org.apache.flex.html.beads.DataGridView.prototype.columns;


/**
 * @expose
 * @return {Array}
 */
org.apache.flex.html.beads.DataGridView.prototype.getColumnLists = function() {
  return this.columns;
};


/**
 * @expose
 * @param {org.apache.flex.core.IStrand} value
 * @override
 */
org.apache.flex.html.beads.DataGridView.prototype.set_strand = function(value) {
  org.apache.flex.html.beads.DataGridView.base(this, 'set_strand', value);
  var /** @type {org.apache.flex.core.IDataGridModel} */ sharedModel = org.apache.flex.utils.Language.as(this._strand.getBeadByType(org.apache.flex.core.IBeadModel), org.apache.flex.core.IDataGridModel);
  org.apache.flex.utils.Language.as(sharedModel, org.apache.flex.events.IEventDispatcher, true).addEventListener("dataProviderChanged", goog.bind(this.onDataProviderChanged, this));
  var /** @type {Array} */ columnLabels = new Array();
  var /** @type {Array} */ buttonWidths = new Array();
  for (var /** @type {number} */ i = 0; i < sharedModel.get_columns().length; i++) {
    var /** @type {org.apache.flex.html.supportClasses.DataGridColumn} */ dgc = org.apache.flex.utils.Language.as(sharedModel.get_columns()[i], org.apache.flex.html.supportClasses.DataGridColumn);
    columnLabels.push(dgc.get_label());
    buttonWidths.push(dgc.get_columnWidth());
  }
  var /** @type {org.apache.flex.html.beads.layouts.ButtonBarLayout} */ bblayout = new org.apache.flex.html.beads.layouts.ButtonBarLayout();
  this.buttonBarModel = new org.apache.flex.html.beads.models.ArraySelectionModel();
  this.buttonBarModel.set_dataProvider(columnLabels);
  this.buttonBar = new org.apache.flex.html.ButtonBar();
  this.buttonBar.addBead(this.buttonBarModel);
  this.buttonBar.addBead(bblayout);
  org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true).addElement(this.buttonBar);
  this.columns = new Array();
  for (i = 0; i < sharedModel.get_columns().length; i++) {
    var /** @type {org.apache.flex.core.ISelectionModel} */ listModel = new org.apache.flex.html.beads.models.ArraySelectionModel();
    listModel.set_dataProvider(sharedModel.get_dataProvider());
    var /** @type {org.apache.flex.html.supportClasses.DataGridColumn} */ dataGridColumn = org.apache.flex.utils.Language.as(sharedModel.get_columns()[i], org.apache.flex.html.supportClasses.DataGridColumn);
    var /** @type {org.apache.flex.html.List} */ list = new org.apache.flex.html.List();
    list.addBead(listModel);
    list.set_itemRenderer(dataGridColumn.get_itemRenderer());
    list.set_labelField(dataGridColumn.get_dataField());
    var /** @type {number} */ colWidth = dataGridColumn.get_columnWidth();
    if (!isNaN(colWidth))
      list.set_width(colWidth);
    org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true).addElement(list);
    this.columns.push(list);
    list.addEventListener('change', goog.bind(this.columnListChangeHandler, this));
    list.addEventListener('rollover', goog.bind(this.columnListRollOverHandler, this));
  }
  var /** @type {org.apache.flex.core.IBead} */ bead = this._strand.getBeadByType(org.apache.flex.core.IDataGridLayout);
  var /** @type {org.apache.flex.core.IDataGridLayout} */ layout;
  if (bead == null) {
    layout = new org.apache.flex.html.beads.layouts.DataGridLayout();
    this._strand.addBead(layout);
  } else {
    layout = org.apache.flex.utils.Language.as(bead, org.apache.flex.core.IDataGridLayout);
  }
  layout.set_header(this.buttonBar);
  layout.set_columns(this.columns);
  org.apache.flex.utils.Language.as(this._strand, org.apache.flex.events.IEventDispatcher, true).dispatchEvent(new org.apache.flex.events.Event("itemsCreated"));
};


/**
 * @private
 * @param {org.apache.flex.events.Event} event
 */
org.apache.flex.html.beads.DataGridView.prototype.onDataProviderChanged = function(event) {
  var /** @type {org.apache.flex.core.IDataGridModel} */ sharedModel = org.apache.flex.utils.Language.as(this._strand.getBeadByType(org.apache.flex.core.IBeadModel), org.apache.flex.core.IDataGridModel);
  for (var /** @type {number} */ i = 0; i < this.columns.length; i++) {
    var /** @type {org.apache.flex.html.List} */ list = org.apache.flex.utils.Language.as(this.columns[i], org.apache.flex.html.List);
    var /** @type {org.apache.flex.core.ISelectionModel} */ listModel = org.apache.flex.utils.Language.as(list.getBeadByType(org.apache.flex.core.IBeadModel), org.apache.flex.core.ISelectionModel);
    listModel.set_dataProvider(sharedModel.get_dataProvider());
  }
};


/**
 * @private
 * @param {org.apache.flex.events.Event} event
 */
org.apache.flex.html.beads.DataGridView.prototype.columnListChangeHandler = function(event) {
  var /** @type {org.apache.flex.core.IDataGridModel} */ sharedModel = org.apache.flex.utils.Language.as(this._strand.getBeadByType(org.apache.flex.core.IBeadModel), org.apache.flex.core.IDataGridModel);
  var /** @type {org.apache.flex.html.List} */ list = org.apache.flex.utils.Language.as(event.get_target(), org.apache.flex.html.List);
  sharedModel.set_selectedIndex(list.get_selectedIndex());
  for (var /** @type {number} */ i = 0; i < this.columns.length; i++) {
    if (list != this.columns[i]) {
      var /** @type {org.apache.flex.html.List} */ otherList = org.apache.flex.utils.Language.as(this.columns[i], org.apache.flex.html.List);
      otherList.set_selectedIndex(list.get_selectedIndex());
    }
  }
  org.apache.flex.utils.Language.as(this._strand, org.apache.flex.events.IEventDispatcher, true).dispatchEvent(new org.apache.flex.events.Event('change'));
};


/**
 * @private
 * @param {org.apache.flex.events.Event} event
 */
org.apache.flex.html.beads.DataGridView.prototype.columnListRollOverHandler = function(event) {
  var /** @type {org.apache.flex.html.List} */ list = org.apache.flex.utils.Language.as(event.get_target(), org.apache.flex.html.List);
  if (list == null)
    return;
  for (var /** @type {number} */ i = 0; i < this.columns.length; i++) {
    if (list != this.columns[i]) {
      var /** @type {org.apache.flex.html.List} */ otherList = org.apache.flex.utils.Language.as(this.columns[i], org.apache.flex.html.List);
      otherList.set_rollOverIndex(list.get_rollOverIndex());
    }
  }
  org.apache.flex.utils.Language.as(this._strand, org.apache.flex.events.IEventDispatcher, true).dispatchEvent(new org.apache.flex.events.Event('rollOver'));
};


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
org.apache.flex.html.beads.DataGridView.prototype.FLEXJS_CLASS_INFO = { names: [{ name: 'DataGridView', qName: 'org.apache.flex.html.beads.DataGridView'}], interfaces: [org.apache.flex.html.beads.IDataGridView] };
