/**
 * org.apache.flex.html.beads.DateFieldView
 *
 * @fileoverview
 *
 * @suppress {checkTypes}
 */

goog.provide('org.apache.flex.html.beads.DateFieldView');

goog.require('org.apache.flex.core.BeadViewBase');
goog.require('org.apache.flex.utils.UIUtils');
goog.require('org.apache.flex.html.TextInput');
goog.require('org.apache.flex.html.DateChooser');
goog.require('org.apache.flex.core.UIBase');
goog.require('org.apache.flex.html.TextButton');
goog.require('org.apache.flex.events.Event');
goog.require('org.apache.flex.events.IEventDispatcher');
goog.require('org.apache.flex.core.IBeadView');
goog.require('org.apache.flex.core.IFormatBead');
goog.require('org.apache.flex.core.IStrand');
goog.require('org.apache.flex.core.IDateChooserModel');
goog.require('org.apache.flex.core.IPopUpHost');



/**
 * @constructor
 * @extends {org.apache.flex.core.BeadViewBase}
 * @implements {org.apache.flex.core.IBeadView}
 */
org.apache.flex.html.beads.DateFieldView = function() {
  org.apache.flex.html.beads.DateFieldView.base(this, 'constructor');
};
goog.inherits(org.apache.flex.html.beads.DateFieldView, org.apache.flex.core.BeadViewBase);


/**
 * @private
 * @type {org.apache.flex.html.TextInput}
 */
org.apache.flex.html.beads.DateFieldView.prototype._textInput;


/**
 * @private
 * @type {org.apache.flex.html.TextButton}
 */
org.apache.flex.html.beads.DateFieldView.prototype._button;


/**
 * @expose
 * @return {org.apache.flex.html.beads.TextButton}
 */
org.apache.flex.html.beads.DateFieldView.prototype.get_menuButton = function() {
  return this._button;
};


/**
 * @expose
 * @return {org.apache.flex.html.beads.TextInput}
 */
org.apache.flex.html.beads.DateFieldView.prototype.get_textInput = function() {
  return this._textInput;
};


/**
 * @expose
 * @param {org.apache.flex.core.IStrand} value
 * @override
 */
org.apache.flex.html.beads.DateFieldView.prototype.set_strand = function(value) {
  org.apache.flex.html.beads.DateFieldView.base(this, 'set_strand', value);
  this._textInput = new org.apache.flex.html.TextInput();
  org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true).addElement(this._textInput);
  this._textInput.set_width(100);
  this._textInput.set_height(18);
  this._button = new org.apache.flex.html.TextButton();
  this._button.set_text("M");
  org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true).addElement(this._button);
  this._button.set_x(this._textInput.get_width());
  this._button.set_y(this._textInput.get_y());
  org.apache.flex.utils.Language.as(this._strand, org.apache.flex.events.IEventDispatcher, true).addEventListener("beadsAdded", goog.bind(this.handleBeadsAdded, this));
};


/**
 * @private
 * @param {org.apache.flex.events.Event} event
 */
org.apache.flex.html.beads.DateFieldView.prototype.handleBeadsAdded = function(event) {
  var /** @type {org.apache.flex.core.IFormatBead} */ formatter = org.apache.flex.utils.Language.as(this._strand.getBeadByType(org.apache.flex.core.IFormatBead), org.apache.flex.core.IFormatBead);
  formatter.addEventListener("formatChanged", goog.bind(this.handleFormatChanged, this));
};


/**
 * @private
 * @param {org.apache.flex.events.Event} event
 */
org.apache.flex.html.beads.DateFieldView.prototype.handleFormatChanged = function(event) {
  var /** @type {org.apache.flex.core.IFormatBead} */ formatter = org.apache.flex.utils.Language.as(event.get_target(), org.apache.flex.core.IFormatBead);
  this._textInput.set_text(formatter.get_formattedString());
};


/**
 * @private
 * @type {org.apache.flex.html.DateChooser}
 */
org.apache.flex.html.beads.DateFieldView.prototype._popUp;


/**
 * @expose
 * @return {org.apache.flex.html.beads.DateChooser}
 */
org.apache.flex.html.beads.DateFieldView.prototype.get_popUp = function() {
  return this._popUp;
};


/**
 * @private
 * @type {boolean}
 */
org.apache.flex.html.beads.DateFieldView.prototype._popUpVisible;


/**
 * @expose
 * @return {boolean}
 */
org.apache.flex.html.beads.DateFieldView.prototype.get_popUpVisible = function() {
  return this._popUpVisible;
};


/**
 * @expose
 * @param {boolean} value
 */
org.apache.flex.html.beads.DateFieldView.prototype.set_popUpVisible = function(value) {
  if (value != this._popUpVisible) {
    this._popUpVisible = value;
    if (value) {
      if (!this._popUp) {
        this._popUp = new org.apache.flex.html.DateChooser();
        this._popUp.set_width(210);
        this._popUp.set_height(220);
        this._popUp.set_x(org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true).get_x());
        this._popUp.set_y(org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true).get_y() + 30);
      }
      var /** @type {org.apache.flex.core.IDateChooserModel} */ model = org.apache.flex.utils.Language.as(this._strand.getBeadByType(org.apache.flex.core.IDateChooserModel), org.apache.flex.core.IDateChooserModel);
      this._popUp.set_selectedDate(model.get_selectedDate());
      var /** @type {org.apache.flex.core.IPopUpHost} */ host = org.apache.flex.utils.UIUtils.findPopUpHost(org.apache.flex.utils.Language.as(this._strand, org.apache.flex.core.UIBase, true));
      host.addElement(this._popUp);
    } else {
      org.apache.flex.utils.UIUtils.removePopUp(this._popUp);
    }
  }
};


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
org.apache.flex.html.beads.DateFieldView.prototype.FLEXJS_CLASS_INFO = { names: [{ name: 'DateFieldView', qName: 'org.apache.flex.html.beads.DateFieldView'}], interfaces: [org.apache.flex.core.IBeadView] };
